@@ -4,46 +4,25 @@
 Helper utils
 '''
 
-import ast
 import inspect
 import linecache
 import pprint
-import textwrap
 
 from . import structure
 
-def get_module_context(obj):
-    '''
-    Instead of providing the whole module source, try to provide a valid
-    context by looking for a smallest chunk of code that can be compiled
-    '''
-    filename = inspect.getsourcefile(obj)
-    linecache.checkcache(filename)
-    lineno = inspect.getlineno(obj)
-    code = linecache.getline(filename, lineno, obj.f_globals)
-    lines = 0
-    while True:
-        try:
-            ast.parse(textwrap.dedent(code) + '\n')
-        except SyntaxError:
-            if not lineno or lines > 10:
-                return code
-            lines += 1
-            prev_line = linecache.getline(filename, lineno - lines,
-                                          obj.f_globals)
-            code = prev_line + code
-        else:
-            return textwrap.dedent(code.strip('\n\r'))
-
 def get_source(obj):
     '''
     Get the source code for the frame object
     '''
-    lines, lineno = inspect.findsource(obj)
-    if inspect.isframe(obj) and obj.f_globals is obj.f_locals:
-        return get_module_context(obj)
-    return textwrap.dedent(
-        ''.join(inspect.getblock(lines[lineno:])).strip('\n\r'))
+    filename = inspect.getsourcefile(obj)
+    linecache.checkcache(filename)
+    lineno = inspect.getlineno(obj)
+    prefix = [(linecache.getline(filename, ln) or u'~').strip('\r\n')
+              for ln in range(lineno-3, lineno)]
+    current = linecache.getline(filename, lineno).strip('\r\n')
+    suffix = [(linecache.getline(filename, ln) or u'~').strip('\r\n')
+              for ln in range(lineno+1, lineno+4)]
+    return prefix, current, suffix
 
 def log(logger, info, indent=0):
     '''
@@ -65,7 +44,7 @@ def log_call(logger, frame, indent=0):
     '''
     filename = inspect.getsourcefile(frame)
     log(logger,
-        structure.CodeLine(filename, frame.f_lineno),
+        structure.FileReference(filename, frame.f_lineno, frame.f_code.co_name),
         indent=indent)
 
 def log_invocation(logger, frame, indent=0):
@@ -87,10 +66,10 @@ def log_frame(logger, frame, indent=0):
     Parse and log a single frame of a traceback
     '''
     log_call(logger, frame, indent=indent)
-    code = get_source(frame)
+    prefix, line, suffix = get_source(frame)
     missing = object()
-    if code:
-        log(logger, structure.Code(code), indent=indent+1)
+    if line:
+        log(logger, structure.Code(prefix, line, suffix), indent=indent+1)
         for key in sorted(frame.f_code.co_varnames):
             value = frame.f_locals.get(
                 key,
